# © 2023 ANSYS, Inc. All rights reserved

"""Module for the gRPC connection object."""
from ansys.sherlock.core.analysis import Analysis
from ansys.sherlock.core.common import Common
from ansys.sherlock.core.layer import Layer
from ansys.sherlock.core.lifecycle import Lifecycle
from ansys.sherlock.core.model import Model
from ansys.sherlock.core.parts import Parts
from ansys.sherlock.core.project import Project
from ansys.sherlock.core.stackup import Stackup


class Sherlock:
    """Sherlock gRPC connection object."""

    def __init__(self, channel):
        """Initialize Sherlock gRPC connection object."""
        self.common = Common(channel)
        self.model = Model(channel)
        self.project = Project(channel)
        self.lifecycle = Lifecycle(channel)
        self.layer = Layer(channel)
        self.stackup = Stackup(channel)
        self.parts = Parts(channel)
        self.analysis = Analysis(channel)



# © 2024 ANSYS, Inc. All rights reserved.

"""Module containing types for the Project Service."""

try:
    import SherlockProjectService_pb2
except ModuleNotFoundError:
    from ansys.api.sherlock.v0 import SherlockProjectService_pb2

project_service = SherlockProjectService_pb2


class BoardBounds:
    """Contains the properties of the board bounds."""

    def __init__(self, bounds):
        """Initialize the board bounds."""
        self.bounds = bounds
        """bounds (two tuples of the form (x, y) : list[tuple[float, float]]"""


class CsvExcelFile:
    """Contains the properties for a thermal map csv or excel file."""

    def __init__(
        self,
        header_row_count,
        numeric_format,
        reference_id_column,
        temperature_column,
        temperature_units,
    ):
        """Initialize the thermal map csv or excel file properties."""
        self.header_row_count = header_row_count
        """header_row_count: int"""
        self.numeric_format = numeric_format
        """numeric_format: string"""
        self.reference_id_column = reference_id_column
        """reference_id_column: string"""
        self.temperature_column = temperature_column
        """temperature_column: string"""
        self.temperature_units = temperature_units
        """temperature_units: string"""


class IcepakFile:
    """Contains the properties for a thermal map Icepak file."""


class ImageBounds:
    """Contains the properties of the image bounds."""

    def __init__(self, image_x, image_y, height, width):
        """Initialize the image bounds properties."""
        self.image_x = image_x
        """x coordinate of the upper left corner : float"""
        self.image_y = image_y
        """y coordinate of the upper left corner : float"""
        self.height = height
        """height of the image : float"""
        self.width = width
        """width of the image : float"""


class ImageFile:
    """Contains the properties for a thermal map image file."""

    def __init__(
        self,
        board_bounds,
        coordinate_units,
        image_bounds,
        legend_bounds,
        legend_orientation,
        max_temperature,
        max_temperature_units,
        min_temperature,
        min_temperature_units,
    ):
        """Initialize the thermal image file properties."""
        self.board_bounds = board_bounds
        """board_bounds : BoardBounds"""
        self.coordinate_units = coordinate_units
        """coordinate_units : string"""
        self.image_bounds = image_bounds
        """image_bounds : ImageBounds"""
        self.legend_bounds = legend_bounds
        """legend_bounds : LegendBounds"""
        self.legend_orientation = legend_orientation
        """legend_orientation : LegendOrientation"""
        self.max_temperature = max_temperature
        """max_temperature : float"""
        self.max_temperature_units = max_temperature_units
        """max_temperature_units : string"""
        self.min_temperature = min_temperature
        """min_temperature : float"""
        self.min_temperature_units = min_temperature_units
        """min_temperature_units : string"""


class LegendBounds:
    """Contains the properties of the legend bounds."""

    def __init__(self, legend_x, legend_y, height, width):
        """Initialize the legend bounds properties."""
        self.legend_x = legend_x
        """x coordinate of the upper left corner : float"""
        self.legend_y = legend_y
        """y coordinate of the upper left corner : float"""
        self.height = height
        """height of the legend : float"""
        self.width = width
        """width of the legend : float"""


class LegendOrientation:
    """Constants for legend orientation in the update thermal maps request."""

    __legend_orientation = project_service.ThermalMapFile.ImageFile.LegendOrientation
    HORIZONTAL = __legend_orientation.Horizontal
    "Horizontal"
    VERTICAL = __legend_orientation.Vertical
    "Vertical"


class ThermalBoardSide:
    """Constants for thermal board side in the update thermal maps request."""

    __thermal_board_side = project_service.ThermalMapFile.ThermalBoardSide
    BOTTOM = __thermal_board_side.Bottom
    "Bottom"
    BOTH = __thermal_board_side.Both
    "Both"
    TOP = __thermal_board_side.Top
    "Top"


class ThermalMapsFileType:
    """Constants for File Type in the Update Thermal Maps request."""

    __file_type = project_service.ThermalMapFile.FileType
    CSV = __file_type.CSV
    "CSV"
    EXCEL = __file_type.Excel
    "Excel"
    IMAGE = __file_type.Image
    "Image"
    TMAP = __file_type.TMAP
    "Icepak Thermal Map (.TMAP)"


class StrainMapsFileType:
    """Constants for File Type in the Add Strain Maps request."""

    __file_type = project_service.AddStrainMapRequest.StrainMapFile.FileType
    CSV = __file_type.CSV
    "CSV"
    EXCEL = __file_type.Excel
    "Excel"
    IMAGE = __file_type.Image
    "Image"


class StrainMapLegendOrientation:
    """Constants for legend orientation in the add strain maps request."""

    __legend_orientation = (
        project_service.AddStrainMapRequest.StrainMapFile.StrainMapImageFile.LegendOrientation
    )
    HORIZONTAL = __legend_orientation.Horizontal
    "Horizontal"
    VERTICAL = __legend_orientation.Vertical
    "Vertical"



# © 2023 ANSYS, Inc. All rights reserved.

"""PySherlock client library."""

# Version
# ------------------------------------------------------------------------------

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:  # pragma: no cover
    import importlib_metadata  # type: ignore

__version__ = importlib_metadata.version(__name__.replace(".", "-"))
"""PySherlock version."""

# Ease import statements
# ------------------------------------------------------------------------------

from ansys.sherlock.core.logging import Logger

LOG = Logger("sherlock")
"""PySherlock logger."""



# © 2023 ANSYS, Inc. All rights reserved.

"""Module containing types for the Layer Service."""


class PolygonalShape:
    """Contains the properties for a polygonal shape."""

    def __init__(self, points, rotation):
        """Initialize the shape properties."""
        self.points = points
        """points (length two tuples of the form (x, y)) : list[tuple[float, float]]"""
        self.rotation = rotation
        """rotation (in degrees) : float"""


class RectangularShape:
    """Contains the properties for a rectangular shape."""

    def __init__(self, length, width, center_x, center_y, rotation):
        """Initialize the shape properties."""
        self.length = length
        """length : float"""
        self.width = width
        """width : float"""
        self.center_x = center_x
        """x coordinate of center : float"""
        self.center_y = center_y
        """y coordinate of center : float"""
        self.rotation = rotation
        """rotation (in degrees) : float"""


class SlotShape:
    """Contains the properties for a slot shape."""

    def __init__(self, length, width, node_count, center_x, center_y, rotation):
        """Initialize the shape properties."""
        self.length = length
        """length : float"""
        self.width = width
        """width : float"""
        self.node_count = node_count
        """node count : int"""
        self.center_x = center_x
        """x coordinate of center : float"""
        self.center_y = center_y
        """y coordinate of center : float"""
        self.rotation = rotation
        """rotation (in degrees) : float"""


class CircularShape:
    """Contains the properties for a circular shape."""

    def __init__(self, diameter, node_count, center_x, center_y, rotation):
        """Initialize the shape properties."""
        self.diameter = diameter
        """diameter : float"""
        self.node_count = node_count
        """node count : int"""
        self.center_x = center_x
        """x coordinate of center : float"""
        self.center_y = center_y
        """y coordinate of center : float"""
        self.rotation = rotation
        """rotation (in degrees) : float"""


class PCBShape:
    """Contains the properties for a PCB shape."""



# © 2023 ANSYS, Inc. All rights reserved.

"""Module containing types for the Parts Service."""

import warnings

try:
    import SherlockCommonService_pb2
    import SherlockPartsService_pb2
except ModuleNotFoundError:
    from ansys.api.sherlock.v0 import SherlockCommonService_pb2, SherlockPartsService_pb2


def deprecation(cls):
    """Raise a DeprecationWarning when a deprecated class is used."""
    # if str(cls).find("PartsListSearchMatchingMode") != -1:
    message = f"{cls} is deprecated. Use a string with the value of the constant name \
    as defined in the proto file."
    warnings.warn(message, DeprecationWarning, stacklevel=2)
    return cls


@deprecation
class PartsListSearchMatchingMode:
    """DEPRECATED. Constants for Matching Mode in Update Parts List & Update Parts from AVL."""

    BOTH = SherlockCommonService_pb2.MatchingMode.Both
    """Both"""
    PART = SherlockCommonService_pb2.MatchingMode.Part
    """Part"""


class PartsListSearchDuplicationMode:
    """Constants for Duplication Mode in Update Parts List and Update Parts from AVL request."""

    FIRST = SherlockPartsService_pb2.DuplicationMode.First
    """First"""
    ERROR = SherlockPartsService_pb2.DuplicationMode.Error
    """Error"""
    IGNORE = SherlockPartsService_pb2.DuplicationMode.Ignore
    """Ignore"""


class AVLPartNum:
    """Constants for AVLPartNum in the Update Parts List from AVL request."""

    ASSIGN_INTERNAL_PART_NUM = SherlockPartsService_pb2.AVLPartNum.AssignInternalPartNum
    """AssignInternalPartNum"""
    ASSIGN_VENDOR_AND_PART_NUM = SherlockPartsService_pb2.AVLPartNum.AssignVendorAndPartNum
    """AssignVendorAndPartNum"""
    DO_NOT_CHANGE_VENDOR_OR_PART_NUM = (
        SherlockPartsService_pb2.AVLPartNum.DoNotChangeVendorOrPartNum
    )
    """DoNotChangeVendorOrPartNum"""


class AVLDescription:
    """Constants for AVLDescription in the Update Parts List from AVL request."""

    ASSIGN_APPROVED_DESCRIPTION = SherlockPartsService_pb2.AVLDescription.AssignApprovedDescription
    """AssignApprovedDescription"""
    DO_NOT_CHANGE_DESCRIPTION = SherlockPartsService_pb2.AVLDescription.DoNotChangeDescription
    """DoNotChangeDescription"""


class PartLocation:
    """Part Location property values."""

    def __init__(self, location):
        """Initialize members from the location."""
        self.x = location.x
        """x coordinate"""
        self.y = location.y
        """y coordinate"""
        self.rotation = location.rotation
        """rotation (in degrees)"""
        self.location_units = location.locationUnits
        """units for location coordinates"""
        self.board_side = location.boardSide
        """board side - ``"TOP"`` or ``"BOTTOM"`` """
        self.mirrored = location.mirrored
        """mirrored - ``True`` or ``False`` """
        self.ref_des = location.refDes
        """reference designator"""
